
let read_range = [-5;-4;-3;-2;-1;0;1;2;3;4;5] 

type var = string 
type program = cmd

and cmd =
	| SKIP
	| IF of cond * cmd * cmd
	| WHILE of cond * cmd 
	| ASSIGN of var * exp
	| READ of var (* assume the external input ranges between -5 and 5 *)
	| SEQ of cmd * cmd
	| PRINT of exp

and exp =
  | CONST of int
  | VAR of var
  | ADD of exp * exp
  | SUB of exp * exp
  | MUL of exp * exp

and cond = 
	| TRUE
	| FALSE
	| LE of var * int
	| EQ of var * int
	| GT of var * int
	| NEQ of var * int
	| NOT of cond

let rec string_of_cmd cmd = 
	match cmd with 
	| SKIP -> "skip"
	| IF (cond, c1, c2) -> Printf.sprintf "if %s then %s else %s" (string_of_cond cond) (string_of_cmd c1) (string_of_cmd c2)
	| WHILE (cond, c) -> Printf.sprintf "while %s { %s }" (string_of_cond cond) (string_of_cmd c)
	| ASSIGN (x, e) -> Printf.sprintf "%s := %s" x (string_of_exp e)
	| READ x -> Printf.sprintf "read %s" x
	| SEQ (c1, c2) -> Printf.sprintf "%s; %s" (string_of_cmd c1) (string_of_cmd c2)
	| PRINT e -> Printf.sprintf "print(%s)" (string_of_exp e)

and string_of_cond cond = 
	match cond with 
	| TRUE -> "true"
	| FALSE -> "false"
	| LE (x, n) -> Printf.sprintf "%s <= %d" x n
	| GT (x, n) -> Printf.sprintf "%s > %d" x n
	| EQ (x, n) -> Printf.sprintf "%s == %d" x n
	| NEQ (x, n) -> Printf.sprintf "%s != %d" x n
	| NOT cond -> "not " ^ (string_of_cond cond)    

and string_of_exp exp = 
	match exp with 
	| CONST n -> string_of_int n
	| VAR x -> x 
	| ADD (e1, e2) -> Printf.sprintf "%s + %s" (string_of_exp e1) (string_of_exp e2)
	| SUB (e1, e2) -> Printf.sprintf "%s - %s" (string_of_exp e1) (string_of_exp e2)
	| MUL (e1, e2) -> Printf.sprintf "%s * %s" (string_of_exp e1) (string_of_exp e2)  