open C
open Domains

let main () =
	  let mode = ref 1 in
		let print_mem = ref false in 
    let src = ref "" in
    let spec = [("-mode", Arg.Int (fun n -> mode := n), "Interpreter based on \n\t1: Operational semantics\n\t2: Collecting semantics\n\t3: Abstract semantics with the parity (even/odd) domain\n\t4: Abstract semantics with the interval domain");
								("-print_mem", Arg.Set print_mem, "Print the final concrete/abstract memory state")
							 ] 
		in
    let usage = "Usage: run <option> <file>" in
    let _ = Arg.parse spec
                (fun
                   x ->
                     if Sys.file_exists x then src := x
                     else raise (Arg.Bad (x ^ ": No files given")))
                usage
    in

  if !src = "" then Arg.usage spec usage
  else
    let file_channel = open_in !src in
    let lexbuf = Lexing.from_channel file_channel in
    let pgm = Parser.program Lexer.start lexbuf in
	  try
			(match !mode with 
			| 1 -> 
				let mem = Interpreter.interpret pgm Mem.empty in 
				(if !print_mem then 
					(print_string "--------- Memory state ---------";
					 (if Mem.is_empty mem then () else print_endline (Mem.string_of_t mem));
					 print_endline "--------------------------------")
				)   
			| 2 -> 
				let mem = Interpreter.interpret_collect pgm IntsetMem.empty in 
				(if !print_mem then 
					(print_string "--------- Collected state ---------";
					 (if IntsetMem.is_empty mem then () else print_endline (IntsetMem.string_of_t mem));
					 print_endline "-----------------------------------")
				) 
			| 3 -> 
				let mem = Interpreter.interpret_parity pgm ParityMem.empty in
				(if !print_mem then 
					(print_string "--------- Abstract state (parity) ---------";
					 (if ParityMem.is_empty mem then () else print_endline (ParityMem.string_of_t mem));
					 print_endline "-------------------------------------------")
				)
			| 4 -> 
				let mem = Interpreter.interpret_interval pgm IntervalMem.empty in
				(if !print_mem then 
					(print_string "--------- Abstract state (interval) ---------";
					 (if IntervalMem.is_empty mem then () else print_endline (IntervalMem.string_of_t mem));
					 print_endline "--------------------------------------------")
				)
			| _ -> print_endline ("Invalid option. The mode should range from 1 to 4.")
			)
  	with Lexer.LexicalError -> print_endline (!src ^ ": Lexical Error")
		
let _ = main ()
